//@ts-ignore
import { Update, UpdateRequest } from "/proto/mmelodies.proto";

// Pre-init protobuf request buffer
const message = UpdateRequest.create({ scopeSamples: true });
const request = UpdateRequest.encode(message).finish();

const dest = document.querySelector("audio#stream");

const canvas = document.getElementById("scope") as HTMLCanvasElement;
const canvasCtx = canvas.getContext("2d") as CanvasRenderingContext2D;

let socket: WebSocket | null = null;

function draw(samples: Float32Array) {
	canvasCtx.fillStyle = "rgb(200, 200, 200)";
	canvasCtx.fillRect(0, 0, canvas.width, canvas.height);

	canvasCtx.lineWidth = 2;
	canvasCtx.strokeStyle = "rgb(0, 0, 0)";

	canvasCtx.beginPath();

	const sliceWidth = (canvas.width * 1.0) / samples.length;
	let x = 0;

	for (let i = 0; i < samples.length; i++) {
		const v = samples[i] + 0.5;
		const y = v * canvas.height;

		if (i === 0) {
			canvasCtx.moveTo(x, y);
		} else {
			canvasCtx.lineTo(x, y);
		}

		x += sliceWidth;
	}

	canvasCtx.lineTo(canvas.width, canvas.height / 2);
	canvasCtx.stroke();
}

// Subscribe to the audioprocess event
function connect_ws() {
	const url = new URL("/api/display", location.href);

	url.protocol = "ws";
	if (location.protocol.startsWith("https")) {
		url.protocol = "wss";
	}

	const ws = new WebSocket(url);
	ws.binaryType = "arraybuffer";

	ws.addEventListener("open", (event) => {
		if (socket) {

			socket.send(request);
		}
	});

	ws.addEventListener("error", (event) => {
		console.log(event);
	});

	ws.addEventListener("close", () => {
		socket = null;
		setTimeout(() => {
			socket = connect_ws();
		}, 2500);
	});

	// Listen for messages
	ws.addEventListener("message", (event) => {
		const message = Update.decode(new Uint8Array(event.data));
		const samples = new Float32Array(message.scopeSamples);

		if (samples.length !== 0) {
			draw(samples);
		}
	});

	return ws;
}

socket = connect_ws();
